/********************************************
 * UTIA AV CR, v.v.i
 * Signal Processing Department
 * Pod Vodarenskou vezi 4
 * 182 08 Prague
 * Czech Republic
 * web: http://zs.utia.cas.cz
 *
 * Filename: main.c
 * Date: 26.03.2016
 * Authors:
 * Jiri Kadlec <kadlec@utia.cas.cz>
 * Zdenek Pohl <xpohl@utia.cas.cz>
 * Lukas Kohout <kohoutl@utia.cas.cz>
 * Description:
 *
 * Changelog:
 *
 *
 ********************************************/

#include <stdio.h>
#include <stdlib.h>

#include <xil_cache.h>
#include <xparameters.h>
#include <platform.h>

#include "ui.h"
#include "img_filters.h"

#include "sds_lib.h"
unsigned long long clock_start, clock_end_processing, clock_end_frame;
#define TIME_STAMP_INIT    clock_start = sds_clock_counter();
#define TIME_STAMP_1  { clock_end_processing = sds_clock_counter();}
#define TIME_STAMP_2  { clock_end_frame = sds_clock_counter();}
#define TIME_STAMP_PRINT {printf("Image processing time: %llu, Total FPS: %f \r\n", clock_end_processing-clock_start, (float)XPAR_CPU_CORTEXA9_0_CPU_CLK_FREQ_HZ/(float)(clock_end_frame-clock_start)); }

void sobel_demo_processing(unsigned short *in_buffer1,
		unsigned short *out_buffer1, unsigned short *in_buffer2,
		unsigned short *out_buffer2, unsigned int j) {
	TIME_STAMP_2
	TIME_STAMP_PRINT
	TIME_STAMP_INIT
	img_process(in_buffer1, out_buffer1, in_buffer2, out_buffer2, j);
	TIME_STAMP_1
}

// ***************************************************************************
// Video Timing Controller VTC
#include "vtc.h"

vtc_t vtc;
extern timing_t vtc_resolutions[VTC_RES_CNT];

// ***************************************************************************
// Video DMA VDMA
#include "video_frame_buffer.h"

#define MAX_RES_X       1920
#define MAX_RES_Y       1200 //1080
#define MAX_FRAME_SIZE  2048*8192              // max size of one frame for VDMA aligned to kB

#define DDR_MEM_BASE  XPAR_DDR_MEM_BASEADDR
#define FB_OFFSET       0x20000000  // 512 MB
#define FB_BASE         DDR_MEM_BASE + FB_OFFSET

video_resolution_t fb_res_in;
video_resolution_t fb_res_out;
video_resolution_t fb_store_res;

/*
 * Frame offset in VDMA
 */
video_offset_t offset = { 0, 0 };

// ***************************************************************************
// IMAGEON
#include <fmc_iic.h>
#include <fmc_imageon.h>
#include "fmc_imageon_iic.h"
#include "fmc_imageon_hdmi_in.h"
#include "fmc_imageon_hdmi_framebuffer.h"

fmc_imageon_hdmi_framebuffer_t fb;

/*
 * EDID content to provide PC input properties to the PC input source
 */
extern u8 utia_hdmii_edid_content[256];

// ***************************************************************************
// Simple hot plug of HDMII
typedef struct {
	unsigned int is_cable_detected;
	unsigned int is_video_valid;
	unsigned int is_unplugged;
} hot_plug_status_t;

hot_plug_status_t hdmii_HPS;

// ***************************************************************************
struct cdce913_pll_conf pc;

#define advanceFStore(f)  {f = (f + 1) % fb.uNumFrames_HdmiFrameBuffer;}

// ***************************************************************************
// ***************************************************************************

// ***************************************************************************
// define black color in YCrCb 4:2:2 color space
#define PIX_Y_INIT     0x00
#define PIX_CBCR_INIT  0x80

void clear_fb_YCrCb(fmc_imageon_hdmi_framebuffer_t *fb) {
	volatile u8 *pStorageMem;
	int i;

	pStorageMem = (u8 *) fb->uBaseAddr_MEM_HdmiFrameBuffer;
	for (i = 0; i < fb->uNumFrames_HdmiFrameBuffer * MAX_FRAME_SIZE;
			i += fb->vdma_rx_bytes) {
		// Init Pixels
		*pStorageMem++ = PIX_Y_INIT;
		*pStorageMem++ = PIX_CBCR_INIT;
	}
	Xil_DCacheFlushRange(fb->uBaseAddr_MEM_HdmiFrameBuffer,
			fb->uNumFrames_HdmiFrameBuffer * MAX_FRAME_SIZE);
}

// ***************************************************************************
int set_resolution(vtc_t *vtc, fmc_imageon_hdmi_framebuffer_t *fb, u8 sel,
		video_resolution_t *res) {
	int status = 0;
	double dummy;

	/*
	 * set new pixel clock
	 */
	xil_printf("Initialize CDCE913 ... ");
	xil_printf(" %d.%d MHz ... ",
			(unsigned int) my_floor(vtc_resolutions[sel].PClk),
			(unsigned int) (my_modf(vtc_resolutions[sel].PClk, &dummy) * 1000.0));
	status = fmc_imageon_vclk_set(&(fb->fmc_imageon),
			vtc_resolutions[sel].PClk);
	if (status != XST_SUCCESS) {
		xil_printf("Failed\r\n");
		return XST_FAILURE;
	} else {
		xil_printf("OK\r\n");
	}

	/*
	 * Initialize VDMA TX
	 */
	xil_printf("Initialize VDMA ... ");
	status = vfb_tx_stop(&fb->vdma_hdmi);
	if (status != XST_SUCCESS) {
		xil_printf("FAILED - Cannot stop VDMA 1 TX\r\n");
		return XST_FAILURE;
	}

	res->x = vtc_resolutions[sel].HActiveVideo;
	res->y = vtc_resolutions[sel].VActiveVideo;

	xil_printf("TX Init ... ");
	status = vfb_tx_init(&fb->vdma_hdmi,                     // pAxiVdma
			&fb->vdmacfg_hdmi_read,             // pReadCfg
			res,                                // uVideoResolution
			&fb_store_res,       // uStorageResolution  -- always set to maximum
			fb->uBaseAddr_MEM_HdmiFrameBuffer,  // uMemAddr
			fb->uNumFrames_HdmiFrameBuffer,     // uNumFrames
			&offset,                           // horizontal and vertical offset
			fb->vdma_tx_bytes                   // number of bytes per pixel
			);
	if (status != XST_SUCCESS) {
		xil_printf("FAILED\n\r");
		return XST_FAILURE;
	} else {
		xil_printf("OK\n\r");
	}

	/*
	 * initialize IMAGEON HDMIO
	 */
	xil_printf("HDMIO Initialization ... ");
	fb->hdmio_timing.IsHDMI = 0;

	fb->hdmio_timing.HActiveVideo = vtc_resolutions[sel].HActiveVideo;
	fb->hdmio_timing.HFrontPorch = vtc_resolutions[sel].HFrontPorch;
	fb->hdmio_timing.HSyncWidth = vtc_resolutions[sel].HSyncWidth;
	fb->hdmio_timing.HBackPorch = vtc_resolutions[sel].HBackPorch;
	fb->hdmio_timing.HSyncPolarity = vtc_resolutions[sel].HSyncPolarity;

	fb->hdmio_timing.VActiveVideo = vtc_resolutions[sel].VActiveVideo;
	fb->hdmio_timing.VFrontPorch = vtc_resolutions[sel].VFrontPorch;
	fb->hdmio_timing.VSyncWidth = vtc_resolutions[sel].VSyncWidth;
	fb->hdmio_timing.VBackPorch = vtc_resolutions[sel].VBackPorch;
	fb->hdmio_timing.VSyncPolarity = vtc_resolutions[sel].VSyncPolarity;

	status = fmc_imageon_hdmio_init(&(fb->fmc_imageon), 1,    // hdmioEnable = 1
			&(fb->hdmio_timing),  // pTiming
			1                     // waitHPD = 0
			);
	if (!status) // fmc_imageon_hdmio_init return 1 on success
		xil_printf("FAILED\n\r");
	else
		xil_printf("OK\n\r");

	/*
	 * initialize timing controller
	 */
	xil_printf("Initialize Timing Controller ... ");
	vtc->timing_sel = sel;

	XVtc_Disable(&vtc->Vtc);

	fill_vtc_timing(vtc);
	set_vtc_timing(vtc);

	xil_printf(" %s ", vtc_resolutions[sel].pName);

	xil_printf(" OK\n\r");

	return XST_SUCCESS;
}

/*
 * Checks the event cable plugged (PC input) and initialize HDMI input
 */
void check_hdmii_cable_plugged(hot_plug_status_t *hp, fmc_imageon_t *inst,
		u8 edid[256]) {
	unsigned int status;

	if (!hp->is_cable_detected && fmc_imageon_hdmii_cable_detect(inst)) {
		hp->is_cable_detected = 1;
		hp->is_video_valid = 0;

		// Initialize HDMI Input, release reset & set HPD (including EDID content)
		status = fmc_imageon_hdmii_init(inst, 1,      // hdmiiEnable = 0
				1,      // editInit = 1
				edid);
		if (!status) {
			xil_printf("ERROR : Failed to initialize HDMI Input Interface\n\r");
			return;
		}

		xil_printf("HDMII cable plugged\n\r");
	}
}

/*
 * Checks the event cable unplugged (PC input) and initialize HDMI input
 */
void check_hdmii_cable_unplugged(hot_plug_status_t *hp, fmc_imageon_t *inst) {
	if (hp->is_cable_detected && !fmc_imageon_hdmii_cable_detect(inst)) {
		// cable unplugged (not powered)
		hp->is_cable_detected = 0;
		hp->is_video_valid = 0;
		hp->is_unplugged = 1;
		xil_printf("HDMII cable unplugged\n\r");
	}
}

/*
 * Detects change of incomig input signals and trigger the reinitialization of PC input
 */
void check_hdmii_resolution(hot_plug_status_t *hp,
		fmc_imageon_hdmi_framebuffer_t *fb, video_resolution_t *res) {
	if (hp->is_video_valid) {
		(void) fmc_imageon_hdmii_get_timing(&(fb->fmc_imageon),
				&(fb->hdmii_timing));
		fb->hdmii_width = fb->hdmii_timing.HActiveVideo;
		fb->hdmii_height = fb->hdmii_timing.VActiveVideo;
		if ((res->x != fb->hdmii_width) || (res->y != fb->hdmii_height)
				|| (res->hfp != fb->hdmii_timing.HFrontPorch)
				|| (res->vfp != fb->hdmii_timing.VFrontPorch)
				|| (res->hsw != fb->hdmii_timing.HSyncWidth)
				|| (res->vsw != fb->hdmii_timing.VSyncWidth)
				|| (res->hbp != fb->hdmii_timing.HBackPorch)
				|| (res->vbp != fb->hdmii_timing.VBackPorch)
				|| (res->hsp != fb->hdmii_timing.HSyncPolarity)
				|| (res->vsp != fb->hdmii_timing.VSyncPolarity)) {
			hp->is_video_valid = 0;
		}
	}
}

/*
 * Set VDMA input to valid video data from HDMII
 */
void set_valid_hdmii_video_signal(hot_plug_status_t *hp,
		fmc_imageon_hdmi_framebuffer_t *fb, video_resolution_t *res) {
	int status;

	if (hp->is_cable_detected && !hp->is_video_valid) {

		get_hdmii_timing(fb);
		print_hdmii_timing(&(fb->hdmii_timing));

		if (fb->hdmii_timing.IsInterlaced) {
			xil_printf(
					"\n\r\tERROR : Interlaced video signal not supported!\n\r");
			return;
		}

		if (fb->hdmii_timing.IsEncrypted) {
			xil_printf(
					"\n\r\tERROR : HDCP encrypted video signal not supported!\n\r");
			return;
		}

		xil_printf("Initialize VDMA ... ");
		vfb_rx_stop(&(fb->vdma_hdmi));

		// Clear frame stores
		clear_fb_YCrCb(fb);

		// set currently locked resolution
		res->x = fb->hdmii_timing.HActiveVideo;
		res->y = fb->hdmii_timing.VActiveVideo;

		res->hfp = fb->hdmii_timing.HFrontPorch;
		res->hsw = fb->hdmii_timing.HSyncWidth;
		res->hbp = fb->hdmii_timing.HBackPorch;
		res->hsp = fb->hdmii_timing.HSyncPolarity;

		res->vfp = fb->hdmii_timing.VFrontPorch;
		res->vsw = fb->hdmii_timing.VSyncWidth;
		res->vbp = fb->hdmii_timing.VBackPorch;
		res->vsp = fb->hdmii_timing.VSyncPolarity;

		// initialize VDMA input
		xil_printf("RX Init ... ");
		status = vfb_rx_init(&(fb->vdma_hdmi),                   // pAxiVdma
				&(fb->vdmacfg_hdmi_write),          // pWriteCfg
				res,  // uVideoResolution     -- use currently locked resolution
				&fb_store_res,  // uStorageResolution   -- always set to maximum
				fb->uBaseAddr_MEM_HdmiFrameBuffer,  // uMemAddr
				fb->uNumFrames_HdmiFrameBuffer,     // uNumFrames
				fb->vdma_rx_bytes                   // number of bytes per pixel
				);

		if (status != XST_SUCCESS) {
			xil_printf("FAILED\n\r\n\r");
			return;
		} else {
			xil_printf("OK\n\r\n\r");
		}

		// resolution changes have impact on errors flags - clear them
		vfb_clr_errors(&(fb->vdma_hdmi));

		hp->is_video_valid = 1;

	}
}

#include "frame_size.h"
//#include "image_cores.h"

int main() {
	unsigned j=(NUMROWS/4);
	int k=1;


	int status;

	u8 key;
	u8 menu_print_en = 1;
	double dummy;

	init_platform();

#ifdef XPAR_MICROBLAZE_USE_ICACHE
	Xil_ICacheEnable();
#endif
#ifdef XPAR_MICROBLAZE_USE_DCACHE
	Xil_DCacheEnable();
#endif

	xil_printf("\n\r\n\r");
	xil_printf("*****************************\n\r");
	xil_printf("*  Signal Processing Dept., *\n\r");
	xil_printf("*  UTIA AV CR, v.v.i.       *\n\r");
	xil_printf("*  HDMI In/Out IMAGEON      *\n\r");
	xil_printf("*  C:/S/t20i2h1/hio         *\n\r");
	xil_printf("*****************************\n\r");
	xil_printf("\n\r\n\r");

	/*
	 * Initialize timer to support usleep and sleep functions when MicroBlaze is used
	 */
#if defined __MICROBLAZE__
	xil_printf("Initialize timer ... ");
	status = init_timer(XPAR_TMRCTR_0_DEVICE_ID);
	if (status != XST_SUCCESS) {
		xil_printf("Failed\r\n");
		return XST_FAILURE;
	} else {
		xil_printf("OK\r\n");
	}
#endif

	/*
	 * HDMI output - IMAGEON on FMC2
	 */
	fb.uBaseAddr_IIC_FmcImageon = XPAR_FMC2_IMAGEON_IIC_BASEADDR;
	fb.uDeviceId_VTC_HdmiiDetector = 0; // no detector used in vdma branch
	fb.uDeviceId_VTC_HdmioGenerator = 0; // no generator needed in vdma branch
	fb.bVerbose = 0;

	/*
	 * Initialize I2C for FMC2
	 */
	xil_printf("IIC IMAGEON Initialization ... ");
	status = fmc_iic_xps_init(&(fb.fmc_imageon_iic),
			"\tHDMII-IMAGEON IIC Controller", fb.uBaseAddr_IIC_FmcImageon);
	if (!status) {
		xil_printf("FAILED\n\r");
		return XST_FAILURE;
	}

	status = fmc_imageon_init(&(fb.fmc_imageon), "\tHDMII-IMAGEON PC Input",
			&(fb.fmc_imageon_iic));
	if (!status) {  // fmc_imageon_hdmio_init return 1 on success
		xil_printf("FAILED\n\r");
		return XST_FAILURE;
	} else {
		xil_printf("OK\n\r");
	}

	fb.fmc_imageon.bVerbose = fb.bVerbose;

	xil_printf("Initialize CDCE913 ... ");
	fmc_imageon_vclk_init(&(fb.fmc_imageon));
	xil_printf(" %d.%d MHz ... ",
			(unsigned int) my_floor(vtc_resolutions[VTC_1920x1080p60].PClk),
			(unsigned int) (my_modf(vtc_resolutions[VTC_1920x1080p60].PClk,
					&dummy) * 1000.0));
	fmc_imageon_vclk_set(&(fb.fmc_imageon),
			vtc_resolutions[VTC_1920x1080p60].PClk);
	xil_printf("OK\n\r");

	/*
	 * Initialize IMAGEON HDMI Output
	 */
	xil_printf("HDMIO Initialization ... ");
	fb.hdmio_timing.IsHDMI = 0;

	fb.hdmio_timing.HActiveVideo =
			vtc_resolutions[VTC_1920x1080p60].HActiveVideo;
	fb.hdmio_timing.HFrontPorch = vtc_resolutions[VTC_1920x1080p60].HFrontPorch;
	fb.hdmio_timing.HSyncWidth = vtc_resolutions[VTC_1920x1080p60].HSyncWidth;
	fb.hdmio_timing.HBackPorch = vtc_resolutions[VTC_1920x1080p60].HBackPorch;
	fb.hdmio_timing.HSyncPolarity =
			vtc_resolutions[VTC_1920x1080p60].HSyncPolarity;

	fb.hdmio_timing.VActiveVideo =
			vtc_resolutions[VTC_1920x1080p60].VActiveVideo;
	fb.hdmio_timing.VFrontPorch = vtc_resolutions[VTC_1920x1080p60].VFrontPorch;
	fb.hdmio_timing.VSyncWidth = vtc_resolutions[VTC_1920x1080p60].VSyncWidth;
	fb.hdmio_timing.VBackPorch = vtc_resolutions[VTC_1920x1080p60].VBackPorch;
	fb.hdmio_timing.VSyncPolarity =
			vtc_resolutions[VTC_1920x1080p60].VSyncPolarity;

	status = fmc_imageon_hdmio_init(&(fb.fmc_imageon), 1,     // hdmioEnable = 1
			&(fb.hdmio_timing), // pTiming
			1                   // waitHPD = 0
			);
	if (!status) {  // fmc_imageon_hdmio_init return 1 on success
		xil_printf("FAILED\n\r");
		return XST_FAILURE;
	} else {
		xil_printf("OK\n\r");
	}

	/*
	 * initialize timing controller
	 */
	xil_printf("Initialize Timing Controller ... ");
	vtc.timing_sel = VTC_1920x1080p60;

	vtc.VtcCfgPtr = XVtc_LookupConfig(XPAR_VTC_0_DEVICE_ID);
	if (vtc.VtcCfgPtr == NULL) {
		xil_printf("Failed to get VTC config\n\r");
		return XST_FAILURE;
	}

	status = XVtc_CfgInitialize(&vtc.Vtc, vtc.VtcCfgPtr,
			vtc.VtcCfgPtr->BaseAddress);
	if (status != XST_SUCCESS) {
		xil_printf("Failed to init VTC");
		return XST_FAILURE;
	}

	XVtc_Disable(&vtc.Vtc);

	fill_vtc_timing(&vtc);
	set_vtc_timing(&vtc);

	xil_printf(" %s ", vtc_resolutions[VTC_1920x1080p60].pName);

	xil_printf(" OK\n\r");

	/*
	 * initialize VDMA output to display FB
	 */
	xil_printf("Initialize VDMA ... ");

	fb.uDeviceId_VDMA_HdmiFrameBuffer = XPAR_AXIVDMA_0_DEVICE_ID;
	fb.uBaseAddr_MEM_HdmiFrameBuffer = FB_BASE;        // frame storage for vdma
	fb.uNumFrames_HdmiFrameBuffer = XPAR_AXIVDMA_0_NUM_FSTORES;
	fb.vdma_tx_bytes = 2;
	fb.vdma_rx_bytes = 2;

	fb_store_res.x = MAX_RES_X;  // always set to maximum
	fb_store_res.y = MAX_RES_Y;  // always set to maximum
	fb_res_out.x = vtc_resolutions[VTC_1920x1080p60].HActiveVideo;
	fb_res_out.y = vtc_resolutions[VTC_1920x1080p60].VActiveVideo;

	xil_printf("Common Init ... ");
	status = vfb_common_init(fb.uDeviceId_VDMA_HdmiFrameBuffer, // uDeviceId
			&fb.vdma_hdmi                      // pAxiVdma
			);
	if (status != XST_SUCCESS) {
		xil_printf("FAILED\n\r");
		return XST_FAILURE;
	}

	xil_printf("TX Init ... ");
	status = vfb_tx_init(&fb.vdma_hdmi,                    // pAxiVdma
			&fb.vdmacfg_hdmi_read,            // pReadCfg
			&fb_res_out,                          // uVideoResolution
			&fb_store_res,       // uStorageResolution  -- always set to maximum
			fb.uBaseAddr_MEM_HdmiFrameBuffer, // uMemAddr
			fb.uNumFrames_HdmiFrameBuffer,    // uNumFrames
			&offset,                          // horizontal and vertical offset
			fb.vdma_tx_bytes                  // number of output bytes
			);
	if (status != XST_SUCCESS) {
		xil_printf("FAILED\n\r");
		return XST_FAILURE;
	} else {
		xil_printf("OK\n\r");
	}

	/*
	 * clear HDMII flags and params before autodetection
	 */
	hdmii_HPS.is_cable_detected = 0;
	hdmii_HPS.is_video_valid = 0;
	hdmii_HPS.is_unplugged = 0;

	fb_res_in.x = 0;
	fb_res_in.y = 0;
	fb_res_in.hfp = 0;
	fb_res_in.hsw = 0;
	fb_res_in.hbp = 0;
	fb_res_in.hsp = 0;
	fb_res_in.vfp = 0;
	fb_res_in.vsw = 0;
	fb_res_in.vbp = 0;
	fb_res_in.vsp = 0;

	//***********************************************************************************

	while (1) {

		/*
		 * Check input cable plugged
		 */
		check_hdmii_cable_plugged(&hdmii_HPS, &(fb.fmc_imageon),
				utia_hdmii_edid_content);

		/*
		 * Check input cable unplugged
		 */
		check_hdmii_cable_unplugged(&hdmii_HPS, &(fb.fmc_imageon));

		/*
		 * Try to detect runtime resolution change on input inputs
		 */
		check_hdmii_resolution(&hdmii_HPS, &fb, &fb_res_in);

		/*
		 * configure VDMA
		 */
		set_valid_hdmii_video_signal(&hdmii_HPS, &fb, &fb_res_in);

		/* print menu */
		//       print_menu(&menu_print_en);
		/*
		 * GUI - terminal
		 */
		key = 's';                  // ui_getch_nb();
		if (key != 0xff) {

			switch (key) {

			case '1':
				(void) set_resolution(&vtc, &fb, VTC_1920x1080p60, &fb_res_out);
				break;

			case '2':
				(void) set_resolution(&vtc, &fb, VTC_1920x1200p50, &fb_res_out);
				break;

			case '3':
				(void) set_resolution(&vtc, &fb, VTC_1600x1200p50, &fb_res_out);
				break;

			case '4':
				(void) set_resolution(&vtc, &fb, VTC_1680x1050p60, &fb_res_out);
				break;

			case '5':
				(void) set_resolution(&vtc, &fb, VTC_1280x1024p60, &fb_res_out);
				break;

			case '6':
				(void) set_resolution(&vtc, &fb, VTC_1280x720p60, &fb_res_out);
				break;

			case '7':
				(void) set_resolution(&vtc, &fb, VTC_1024x768p60, &fb_res_out);
				break;

			case '8':
				(void) set_resolution(&vtc, &fb, VTC_800x600p60, &fb_res_out);
				break;

			case '9':
				(void) set_resolution(&vtc, &fb, VTC_640x480p60, &fb_res_out);
				break;

			case 'a':
				(void) set_resolution(&vtc, &fb, VTC_600x800p60, &fb_res_out);
				break;

			case 'm':
				menu_print_en = 1;
				break;

			case 'x':
				xil_printf("Exiting\n\r");
				return XST_FAILURE;
				break;

			case 's':
				xil_printf("Running sobel ....\r\n");
				key = ' ';

				// Status of AXI VDMA
				vfb_dump_registers(&fb.vdma_hdmi);
				if (vfb_check_errors(&fb.vdma_hdmi)) //, 1/*clear errors, if any*/ ) )
						{
					vfb_dump_registers(&fb.vdma_hdmi);
				}

				/*
				 * Check available number of framebuffers
				 */
				if (fb.uNumFrames_HdmiFrameBuffer < 8) {
					xil_printf(
							"ERROR; not enough frame buffers to run sobel demo\r\n");
					break;
				}

				unsigned writingFrame = 0; // frame ktery se zapisuje z hdmi vstupu
				unsigned nextInputFrame = fb.uNumFrames_HdmiFrameBuffer - 1; // frame ktery je pripraven az se zacne zpracovavat
				unsigned inputFrame = fb.uNumFrames_HdmiFrameBuffer - 2; // frame ktery se prave zpracovava
				unsigned previousInputFrame = fb.uNumFrames_HdmiFrameBuffer - 3; // frame ktery se zpracovaval predtim
				unsigned outputFrame = fb.uNumFrames_HdmiFrameBuffer - 4; // frame pripraveny na dalsi vystup
				unsigned nextOutputFrame = fb.uNumFrames_HdmiFrameBuffer - 5; // frame do ktereho se zapisuje vysledek zpracovani
				unsigned readingFrame = fb.uNumFrames_HdmiFrameBuffer - 6; // frame ktery se cte do hdmi vystupu
				// setup parking
				XAxiVdma_StartParking(&fb.vdma_hdmi, writingFrame,
						XAXIVDMA_WRITE);
				XAxiVdma_StartParking(&fb.vdma_hdmi, readingFrame,
						XAXIVDMA_READ);

				xil_printf("Parking started\r\n");

				while (1) {
					/*
					 * wait until new storage is used (set before sending previous frame)
					 */
					while (writingFrame
							!= XAxiVdma_CurrFrameStore(&fb.vdma_hdmi,
									XAXIVDMA_WRITE)) {
					};
					while (readingFrame
							!= XAxiVdma_CurrFrameStore(&fb.vdma_hdmi,
									XAXIVDMA_READ)) {
					};

					/*
					 * send previous one and move parking to next
					 */
					advanceFStore(writingFrame);
					advanceFStore(nextInputFrame);
					advanceFStore(inputFrame);
					advanceFStore(previousInputFrame);
					advanceFStore(outputFrame);
					advanceFStore(nextOutputFrame);
					advanceFStore(readingFrame);

					/*
					 * Park to new positions
					 */
					XAxiVdma_StartParking(&fb.vdma_hdmi, readingFrame,
							XAXIVDMA_READ); // set parking to next, do not wait until switch
					XAxiVdma_StartParking(&fb.vdma_hdmi, writingFrame,
							XAXIVDMA_WRITE); // set parking to next, do not wait until switch

					/*
					 * Do all computations over inputFrame, previousInputFrame and outputFrame
					 */
					sobel_demo_processing(
							(unsigned short*) fb.vdmacfg_hdmi_read.FrameStoreStartAddr[inputFrame],
							(unsigned short*) fb.vdmacfg_hdmi_read.FrameStoreStartAddr[outputFrame],
							(unsigned short*) (fb.vdmacfg_hdmi_read.FrameStoreStartAddr[inputFrame]+ NUMROWS*NUMPADCOLS*2),
							(unsigned short*) (fb.vdmacfg_hdmi_read.FrameStoreStartAddr[outputFrame]+ NUMROWS*NUMPADCOLS*2),j);
					if ((j > (NUMROWS/4)-1)&&(j < (NUMROWS))){
						j= j+k;
					}
					else{
						k=k*(-1);
						j=j+k;
					}
				}
				break;
			}
		}

	}

	Xil_DCacheDisable();
	Xil_ICacheDisable();

	cleanup_platform();

	return XST_SUCCESS;
}

